/*
 *  arch-tag: Implementation of new podcast dialog
 *
 *  Copyright (C) 2005 Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnome/gnome-i18n.h>
#include <gtk/gtkentry.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkcombo.h>
#include <gtk/gtkbox.h>
#include <gtk/gtktable.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkmessagedialog.h>
#include <glade/glade.h>
#include <string.h>
#include <time.h>

#include "rb-new-podcast-dialog.h"
#include "rb-glade-helpers.h"
#include "rb-dialog.h"
#include "rb-podcast-download.h"

static void rb_new_podcast_dialog_class_init (RBNewPodcastDialogClass *klass);
static void rb_new_podcast_dialog_init (RBNewPodcastDialog *dialog);
static void rb_new_podcast_dialog_finalize (GObject *object);
static void rb_new_podcast_dialog_set_property (GObject *object,
						guint prop_id,
						const GValue *value,
						GParamSpec *pspec);
static void rb_new_podcast_dialog_get_property (GObject *object,
						guint prop_id,
						GValue *value,
						GParamSpec *pspec);
static void rb_new_podcast_dialog_response_cb (GtkDialog *gtkdialog,
					       int response_id,
					       RBNewPodcastDialog *dialog);
static void rb_new_podcast_dialog_entry_changed_cb (GtkEntry *entry,
						    RBNewPodcastDialog *dialog);
//static gpointer rb_new_podcast_dialog_copy_file(gpointer data);


struct RBNewPodcastDialogPrivate
{
	RhythmDB    *db;

	GtkWidget   *url;
	GtkWidget   *okbutton;
	GtkWidget   *cancelbutton;
};

enum 
{
	PROP_0,
	PROP_DB
};

static GObjectClass *parent_class = NULL;

GType
rb_new_podcast_dialog_get_type (void)
{
	static GType rb_new_podcast_dialog_type = 0;

	if (rb_new_podcast_dialog_type == 0)
	{
		static const GTypeInfo our_info =
		{
			sizeof (RBNewPodcastDialogClass),
			NULL,
			NULL,
			(GClassInitFunc) rb_new_podcast_dialog_class_init,
			NULL,
			NULL,
			sizeof (RBNewPodcastDialog),
			0,
			(GInstanceInitFunc) rb_new_podcast_dialog_init
		};

		rb_new_podcast_dialog_type = g_type_register_static (GTK_TYPE_DIALOG,
								     "RBNewPodcastDialog",
								     &our_info, 0);
	}

	return rb_new_podcast_dialog_type;
}

static void
rb_new_podcast_dialog_class_init (RBNewPodcastDialogClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->set_property = rb_new_podcast_dialog_set_property;
	object_class->get_property = rb_new_podcast_dialog_get_property;

	g_object_class_install_property (object_class,
					 PROP_DB,
					 g_param_spec_object ("db",
					                      "RhythmDB",
					                      "RhythmDB object",
					                      RHYTHMDB_TYPE,
					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));

	object_class->finalize = rb_new_podcast_dialog_finalize;
}

static void
rb_new_podcast_dialog_init (RBNewPodcastDialog *dialog)
{
	GladeXML *xml;

	/* create the dialog and some buttons forward - close */
	dialog->priv = g_new0 (RBNewPodcastDialogPrivate, 1);


	g_signal_connect_object (G_OBJECT (dialog),
				 "response",
				 G_CALLBACK (rb_new_podcast_dialog_response_cb),
				 dialog, 0);

	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);

	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
					 GTK_RESPONSE_OK);
	gtk_window_set_title (GTK_WINDOW (dialog), _("New Podcast"));

	dialog->priv->cancelbutton = gtk_dialog_add_button (GTK_DIALOG (dialog),
							    GTK_STOCK_CANCEL,
							    GTK_RESPONSE_CANCEL);
	dialog->priv->okbutton = gtk_dialog_add_button (GTK_DIALOG (dialog),
							GTK_STOCK_ADD,
							GTK_RESPONSE_OK);
	
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);

	xml = rb_glade_xml_new ("podcast-new.glade",
				"newpodcast",
				dialog);
	glade_xml_signal_autoconnect (xml);

	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
			   glade_xml_get_widget (xml, "newpodcast"));

	
	/* get the widgets from the XML */
	dialog->priv->url = glade_xml_get_widget (xml, "urlEntry");

	g_signal_connect_object (G_OBJECT (dialog->priv->url),
				 "changed",
				 G_CALLBACK (rb_new_podcast_dialog_entry_changed_cb),
				 dialog, 0);


	/* default focus */
	gtk_widget_grab_focus (dialog->priv->url);
	
	/* FIXME */
	gtk_widget_set_sensitive (dialog->priv->okbutton, FALSE);

	g_object_unref (G_OBJECT (xml));
}

static void
rb_new_podcast_dialog_finalize (GObject *object)
{
	RBNewPodcastDialog *dialog;

	g_return_if_fail (object != NULL);
	g_return_if_fail (RB_IS_NEW_PODCAST_DIALOG (object));

	dialog = RB_NEW_PODCAST_DIALOG (object);

	g_return_if_fail (dialog->priv != NULL);

	g_free (dialog->priv);

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
rb_new_podcast_dialog_set_property (GObject *object,
			   guint prop_id,
			   const GValue *value,
			   GParamSpec *pspec)
{
	RBNewPodcastDialog *dialog = RB_NEW_PODCAST_DIALOG (object);

	switch (prop_id)
	{
	case PROP_DB:
		dialog->priv->db = g_value_get_object (value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
rb_new_podcast_dialog_get_property (GObject *object,
			      guint prop_id,
			      GValue *value,
			      GParamSpec *pspec)
{
	RBNewPodcastDialog *dialog = RB_NEW_PODCAST_DIALOG (object);

	switch (prop_id)
	{
	case PROP_DB:
		g_value_set_object (value, dialog->priv->db);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

GtkWidget *
rb_new_podcast_dialog_new (RhythmDB *db)
{
	RBNewPodcastDialog *dialog;
	
	g_return_val_if_fail (RHYTHMDB_IS (db), NULL);
	
	dialog = g_object_new (RB_TYPE_NEW_PODCAST_DIALOG, "db", db, NULL);

	g_return_val_if_fail (dialog->priv != NULL, NULL);

	return GTK_WIDGET (dialog);
}

static void
rb_new_podcast_dialog_response_cb (GtkDialog *gtkdialog,
				   int response_id,
				   RBNewPodcastDialog *dialog)
{
	const gchar *valid_url;
	if (response_id != GTK_RESPONSE_OK)
		return;

	valid_url = gtk_entry_get_text (GTK_ENTRY (dialog->priv->url));

	if (!rb_podcast_download_add_url (dialog->priv->db, valid_url)) {
		GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (gtkdialog),
		                                  	    GTK_DIALOG_MODAL,
						            GTK_MESSAGE_ERROR,
							    GTK_BUTTONS_CLOSE,
							    _("Error loading file"));
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
	}
		
	gtk_widget_hide(GTK_WIDGET(gtkdialog));	
	
}

static void
rb_new_podcast_dialog_entry_changed_cb (GtkEntry *entry,
					RBNewPodcastDialog *dialog)
{
	gtk_widget_set_sensitive (dialog->priv->okbutton,
				  g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY (dialog->priv->url)), -1) > 0);
}
