/*
 *  arch-tag: Header files for podcast download manager
 *
 *  Copyright (C) 2005 Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <glib.h>
#include "rhythmdb.h"

#ifndef RB_PODCAST_DOWNLOAD_H
#define RB_PODCAST_DOWNLOAD_H


G_BEGIN_DECLS

#define RB_TYPE_PODCAST_DOWNLOAD            (rb_podcast_download_get_type ())
#define RB_PODCAST_DOWNLOAD(o)              (G_TYPE_CHECK_INSTANCE_CAST ((o), RB_TYPE_PODCAST_DOWNLOAD, RBPodcastDownload))
#define RB_PODCAST_DOWNLOAD_CLASS(k)        (G_TYPE_CHECK_CLASS_CAST((k), RB_TYPE_PODCAST_DOWNLOAD, RBPodcastDownloadClass))
#define RB_IS_PODCAST_DOWNLOAD(o)           (G_TYPE_CHECK_INSTANCE_TYPE ((o), RB_TYPE_PODCAST_DOWNLOAD))
#define RB_IS_PODCAST_DOWNLOAD_CLASS(k)     (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_PODCAST_DOWNLOAD))
#define RB_PODCAST_DOWNLOAD_GET_CLASS(o)    (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_PODCAST_DOWNLOAD, RBPodcastDownloadClass))

typedef struct RBPodcastDownloadPrivate RBPodcastDownloadPrivate;

typedef struct
{
        GObject parent;

        RBPodcastDownloadPrivate *priv;
} RBPodcastDownload;

typedef struct
{
	GObjectClass parent_class;

    /* signals */
    void        (*status_changed)   (RBPodcastDownload* pd, RhythmDBEntry *entry, glong value);
} RBPodcastDownloadClass;

GType                   rb_podcast_download_get_type    (void);
RBPodcastDownload*      rb_podcast_download_new         (RhythmDB *db);
void                    rb_podcast_download_add_entry   (RBPodcastDownload *pd, RhythmDBEntry *entry);
void                    rb_podcast_download_start_sync  (RBPodcastDownload *pd);
gboolean                rb_podcast_download_add_url     (RhythmDB *db, const gchar* url);
gboolean                rb_podcast_download_remove_url  (RhythmDB *db, const gchar* url);
void                    rb_podcast_download_add_post    (RhythmDB *db,
                               			         const char *name,
	                                                 const char *title,
	                                                 const char *subtitle,
	                                                 const char *generator,
	                                                 const char *uri,
							 gulong status,
	                                                 gulong date);

G_END_DECLS

#endif /* RB_PODCAST_DOWNLOAD_H */
